
from rest_framework import serializers
from clusterCore.models import c123_node, c123_host, c123_cluster, Task
from clusterCore.enums import DBType, ClusterRoles, Inventory, Status, OperatingSystem, Severity, Schedule, TaskState, DataBaseType
from clusterCore.api.serializer import c123_inventory, c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, FORBID_APP_HOST, RECREATION_CONFLICT
from clusterCore.exceptions import *
from clusterCore.tasks import check_os_version, drop_backup, format_rsa_key, license_after_create, get_fully_qualified_domain_name, format_password
import json
from django.utils import timezone

class c123_CreatePostgresPodSerializer(serializers.HyperlinkedModelSerializer):
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'database_user', 'database_pwd']
    
    @license_after_create
    def create(self, validated_data):
        
        create_run_linux = True
        host_data = validated_data.pop('host')
        hostname = host_data['name']
        node_exists = c123_node.objects.filter(name = validated_data['name'])
        if node_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.POSTGRES_K8S.value, validated_data['name']))
        node_to_update = node_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        host_data['ex_params'] = {}
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.POSTGRES_K8S.value
        cluster_data['name'] = validated_data['name']
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value
        validated_data['type'] = DBType.POSTGRES_K8S.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        validated_data["cluster"] = own_cluster
        # validated_data['database_pwd'] = format_password(validated_data['database_pwd'])
        validated_data['ind_deleted'] = False
         
        private_key = format_rsa_key(host_data['private_key'].strip())
        host_exist = c123_host.objects.filter(name = hostname).exists()     
        if not host_exist:
            host = c123_host.objects.create(**host_data)
        else:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        if node_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=node_to_update.pk).update(**validated_data)
            node = c123_node.objects.get(pk=node_to_update.pk)         
        else:    
            node = c123_node.objects.create(**validated_data)
        node.save()
        if self.context["request"].data.get("auto_delete"):
            node.auto_delete =  node.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            node.save()
        
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = node))
        else:
            c123_inventory.objects.filter(node = node).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = node.type, ex_params__node_version = node.version).delete()
        tmp_data = json.dumps({'node_data': object_to_json(node), 'host_data': object_to_json(node.host), 'create_run_linux': create_run_linux, 'created_by': current_user.id})
        task = Task.objects.create(task="install_k8s_postgres_standalone_task",
                                        displayment='creating postgresql pod {}'.format(node.name),
                                        target_type=DBType.POSTGRES_K8S.value,
                                        target_id=node.id,
                                        org_id=node.org_id,
                                        status=TaskState.PENDING.value,
                                        schedule=Schedule.NOW.value,
                                        parameters=tmp_data,
                                        created_by=current_user)
        return node, task